/*
半田ごてタイマープログラム
*/
#include "SPI.h"

#define TIMER_COUNT 60    //タイマー時間
#define TIMER_ALART_COUNT TIMER_COUNT-5

#define TEMP_HI  250   //上限温度
#define TEMP_LO  230  //加熱復帰温度

#define VCC  8
#define GND  9
#define SLAVE  10

const int startbuttonPin = 5;     // Timer start button
const int stopbuttonPin = 4;     // Timer stop button
const int ledPin =  6;      // Status LED pin
const int ac_ctrlPin =  7;      // AC ctrl signal pin
const int BZ_outPin =  3;      // AC ctrl signal pin

// variables will change:
int PowerState = 0;         // variable for reading the pushbutton status
int timer_count_min;
int timer_count_msec;
int timer_count_sec;

void setup() {
  
//温度モジュールのための電源確保
  pinMode(GND, OUTPUT);
  digitalWrite(GND, LOW); 
  pinMode(VCC, OUTPUT);
  digitalWrite(VCC, HIGH);
  pinMode(SLAVE, OUTPUT);
  digitalWrite(SLAVE, HIGH);

//SPI初期化
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV4);
  SPI.setDataMode(SPI_MODE0);
  
  // ＩＯ初期化
  pinMode(ledPin, OUTPUT);      
  pinMode(ac_ctrlPin, OUTPUT);      
  pinMode(startbuttonPin, INPUT_PULLUP);     
  pinMode(stopbuttonPin, INPUT_PULLUP);     
  digitalWrite(ledPin, LOW);  
  digitalWrite(ac_ctrlPin, LOW);  
  timer_count_min=0;
  timer_count_sec=0;
  PowerState=0;
}


void loop(){
  int value,temp;
  
  while(PowerState!=1)  //スタンバイ時タスク 本来==0で良いが、01以外で何も機能しなくなるため保険
  {
    // スタートボタンが押された
    if (digitalRead(startbuttonPin) == LOW) {     
      // ＬＥＤつける:    
      PowerState=1;
      digitalWrite(ledPin, HIGH);  
      digitalWrite(ac_ctrlPin, HIGH);
      tone(BZ_outPin, 1000,50);
      delay(50);
      tone(BZ_outPin, 2000,50);
        // カウンタ初期化
      timer_count_min=0;
      timer_count_sec=0;
      timer_count_msec=0;
    } 
  }
  
  while(PowerState==1) //通電時タスク
  {
    delay(100);
    timer_count_msec++;
    if(timer_count_min>=TIMER_ALART_COUNT) //5分前にLEDを点滅させる
    {
        if (digitalRead(startbuttonPin) == LOW) { //5分前にボタンが押されたら延長
          timer_count_min=0;
        }
       if (digitalRead(ledPin) == HIGH) // LED状態反転
          digitalWrite(ledPin, LOW);
       else
          digitalWrite(ledPin, HIGH);
    }
    if(timer_count_msec>=10)  //一秒置きの処理
    {
      timer_count_msec=0;
      timer_count_sec++;

      //温度取得
      digitalWrite(SLAVE, LOW);        //  Enable a chip
      value = SPI.transfer(0x00) << 8;  //  read High byte
      value |= SPI.transfer(0x00);      //  read Low byte
      digitalWrite(SLAVE, HIGH);         //  Desavle a chip
      if ((value & 0x0004) == 0){
        temp=(value >> 3) * 0.25;
        if(temp>TEMP_HI)    //基準温度を超えるとOFFにする
        {
          digitalWrite(ac_ctrlPin, LOW); 
        }
        if(temp<TEMP_LO)    //基準温度を下回るとONにする
        {
          digitalWrite(ac_ctrlPin, HIGH); 
        }
      } 
    
      if(timer_count_sec>=60)  //1分置きの処理
      {
        timer_count_sec=0;
        timer_count_min++;
        if(timer_count_min==TIMER_ALART_COUNT) //5分前にブザーを鳴らす
        {
          tone(BZ_outPin, 2000,50);
          delay(100);
          tone(BZ_outPin, 2000,50);         
          delay(100);
          tone(BZ_outPin, 2000,50);         
        }
        if(timer_count_min>TIMER_COUNT)
        {
            timer_count_min=0;
            PowerState=0;
            digitalWrite(ledPin, LOW); 
            digitalWrite(ac_ctrlPin, LOW); 
            tone(BZ_outPin, 2000,50);
            delay(50);
            tone(BZ_outPin, 1000,50);
        }
      }
    }

    // ストップボタンが押された
    if (digitalRead(stopbuttonPin) == LOW) {     
      // ＬＥＤけす:
      PowerState=0;
      digitalWrite(ledPin, LOW); 
      digitalWrite(ac_ctrlPin, LOW); 
      tone(BZ_outPin, 2000,50);
      delay(50);
      tone(BZ_outPin, 1000,50);
    }
  }  
} 

